@@ -7,7 +7,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "@(#)$Id: commands.c,v 1.52 1998/06/25 15:54:21 kalt Exp $";
+static char rcsid[] = "@(#)$Id: commands.c,v 1.55 1998/09/03 20:38:07 kalt Exp $";
 #endif
 
 #include "os.h"
@@ -53,6 +53,7 @@ static struct aliaslist_ *alias = NULL;
  *	-2	"no recipient"
  *	-3	"no text to send"
  *	-4	"bad params"
+ *	-5	"bind window to server first"
  *	-9	"not implemented"
  */
 
@@ -134,7 +135,10 @@ cmd_join(p)
 
   if (!*p)
       return -1;
-
+  if (index(p, ','))
+      return -9; /* better this than a crash */
+  if (server == NULL)
+      return -5;
   if (key)
       *key = '\0';
   switch (map_c2w(p))
@@ -248,6 +252,8 @@ char *cmd, *p;
 	}
     }
 
+  if (*p != '=' && server == NULL)
+      return -5;
   fmt = get_channel(p) ? (*cmd == 'P') ? F_MYPUBM : F_MYPUBN :
       (*cmd == 'D') ? F_MYDCC : (*cmd == 'P') ? F_MYPRIVM : F_MYPRIVN;
   rendering = out_sprintf(outp, outa, get_format(fmt, p), p, cmd, txt);
@@ -325,6 +331,8 @@ cmd_squery(p)
 
   if (!*p)
       return -2;
+  if (server == NULL)
+      return -5;
   if (!txt || !*(txt+1))
       return -3;
   *txt++ = '\0';
@@ -359,6 +367,8 @@ static int
 cmd_umode(p)
   char *p;
 {
+  if (server == NULL)
+      return -5;
   vsic_write("MODE %s %s", server->nick, p);
   return 0;
 }
@@ -439,6 +449,7 @@ cmd_clear(p)
       sic_clog(1);
   else
       sic_clog(0);
+  sic_scroll2(+1, NULL);
   sic_redowin(0);
   return 0;
 }
@@ -740,58 +751,60 @@ cmd_window(p)
 }
 
 struct cmdlist_	cmdlist[] = {
-	{ "ADMIN",	NULL},
+	{ "ADMIN",	NULL },
 	{ "ALIAS",	cmd_alias},
-	{ "CLOSE",	NULL},
-	{ "CONNECT",	NULL},
+	{ "AWAY",	NULL },
+	{ "CLOSE",	NULL },
+	{ "CONNECT",	NULL },
 	{ "CLEAR",	cmd_clear},
 	{ "CMODE",	cmd_cmode},
 	{ "CTCP",	cmd_ctcp},
 	{ "DCC",	cmd_dcc},
-	{ "DIE",	NULL},
+	{ "DIE",	NULL },
 	{ "DNS",	cmd_dns},
 	{ "EXIT",	cmd_exit},
-	{ "INFO",	NULL},
-	{ "INVITE",	NULL},
-	{ "ISON",	NULL},
+	{ "INFO",	NULL },
+	{ "INVITE",	NULL },
+	{ "ISON",	NULL },
 	{ "JOIN",	cmd_join },
 	{ "KICK",	cmd_kick},
-	/*{ "KILL",	NULL},*/
+	/*{ "KILL",	NULL },*/
 	{ "LASTLOG",	cmd_lastlog},
 	{ "LEAVE",	cmd_leave},
-	{ "LINKS",	NULL},
+	{ "LINKS",	NULL },
 	{ "LIST",	cmd_list},
-	{ "LUSERS",	NULL},
-	{ "MOTD",	NULL},
-	{ "MODE",	NULL},
+	{ "LUSERS",	NULL },
+	{ "MOTD",	NULL },
+	{ "MODE",	NULL },
 	{ "MSG",	cmd_msg},
 	{ "NAMES",	cmd_names},
-	{ "NICK",	NULL},
+	{ "NICK",	NULL },
 	{ "NOTICE",	cmd_notice},
-	{ "OPER",	NULL},
+	{ "OPER",	NULL },
 	{ "OPTION",	cmd_option},
 	{ "PART",	cmd_part},
 	{ "PING",	cmd_ping},
 	/* PING/PONG */
 	{ "QUERY",	cmd_query},
 	{ "QUIT",	cmd_quit},
-	{ "REHASH",	NULL},
+	{ "REHASH",	NULL },
+	{ "RESTART",	NULL },
 	{ "SEEN",	cmd_seen},
 	{ "SERVER",	cmd_server},
-	{ "SERVLIST",	NULL},
+	{ "SERVLIST",	NULL },
 	{ "SQUERY",	cmd_squery},
-	/*{ "SQUIT",	NULL},*/
-	{ "STATS",	NULL},
-	{ "TIME",	NULL},
+	/*{ "SQUIT",	NULL },*/
+	{ "STATS",	NULL },
+	{ "TIME",	NULL },
 	{ "TOPIC",	cmd_topic },
-	{ "TRACE",	NULL},
+	{ "TRACE",	NULL },
 	{ "UMODE",	cmd_umode},
-	{ "USERHOST",	NULL},
-	{ "VERSION",	NULL},
+	{ "USERHOST",	NULL },
+	{ "VERSION",	NULL },
 	{ "WINDOW",	cmd_window },
 	{ "WHO",	cmd_who },
-	{ "WHOIS",	NULL},
-	{ "WHOWAS",	NULL},
+	{ "WHOIS",	NULL },
+	{ "WHOWAS",	NULL },
 	{ NULL,		NULL }
 	};
 
@@ -941,6 +954,10 @@ parse_command(str, aliases)
 	      vsic_slog(LOG_CLIENT, "%s: Invalid parameter(s).",
 			cmdlist[i].name);
 	      break;
+	  case -5:
+	      vsic_slog(LOG_CLIENT, "%s: Bind the window to a server first!",
+			cmdlist[i].name);
+	      break;
 	  case -9:
 	      vsic_slog(LOG_CLIENT, "%s: functionality not implemented.",
 			cmdlist[i].name);
