@@ -7,7 +7,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "@(#)$Id: server.c,v 1.51 1998/06/25 18:15:56 kalt Exp $";
+static char rcsid[] = "@(#)$Id: server.c,v 1.56 1998/08/31 14:52:29 kalt Exp $";
 #endif
 
 #include "os.h"
@@ -324,6 +324,7 @@ sic_connect(srv)
   unset_option(srv->sopt, S_QUIT);
   srv->fd = fd;
   srv->lastr = 0;
+  srv->readbuf[0] = '\0';
 }
 
 /* sic_listen: create a new socket, and bind it */
@@ -505,18 +506,21 @@ readln(fd, buffer, length)
 {
   int status, ptr;
 
-  ptr = 0;
+  ptr = -1;
+  while (buffer[++ptr]);
 
-  do
+  while (((status = read(fd, &buffer[ptr], 1)) > 0) && (ptr < length - 1))
     {
-      while (((status = read(fd, &buffer[ptr], 1)) > 0) && (ptr < length - 1))
-	{
-	  if (buffer[ptr++] == '\n')
-	      break;
-	}
+      if (buffer[ptr++] == '\n')
+	  break;
+    }
+
+  if (status == -1 && (errno == EWOULDBLOCK || errno == EAGAIN))
+    {
+      /* incomplete line */
+      buffer[ptr] = '\0';
+      return -2;
     }
-  while (status == -1 && (errno == EWOULDBLOCK || errno == EAGAIN));
-  /* this isn't non-blocking! */
 
   if (status > 0)
     {
@@ -632,14 +636,23 @@ sic_select()
 	{
 	  if (!option(stmp->sopt, S_DCC))
 	    {
+	      server = stmp;
+	      select_active(NULL, 0);
 	      if (stmp->lastr && time(NULL) - stmp->lastr > 60)
-		  write(stmp->fd, "PING .\r\n", 8);
-	      if (need_collect)
 		{
-		  server = stmp;
-		  select_active(NULL, 0); /* default window for this server */
-		  collect_channel();
+		  if ((time(NULL) - stmp->lastr) % 120 < 60)
+		    {
+		      if (!option(stmp->sopt, S_PING))
+			{
+			  vsic_write("PING .");
+			  set_option(stmp->sopt, S_PING);
+			}
+		    }
+		  else
+		      unset_option(stmp->sopt, S_PING);
 		}
+	      if (need_collect)
+		  collect_channel();
 	    }
 	  FD_SET(stmp->fd, &rfd);
 	}
@@ -783,7 +796,11 @@ sic_select()
 	      if (option(stmp->sopt, S_DCC) && stmp->pass)
 		  sz = read(stmp->fd, buf, 8192); /* big buffer for files */
 	      else
-		  sz = readln(stmp->fd, buf, 600);/*read up to \r\n otherwise*/
+		{
+		  /*read up to \r\n otherwise*/
+		  unset_option(stmp->sopt, S_PING);
+		  sz = readln(stmp->fd, stmp->readbuf, 520);
+		}
 	      if (sz > 0)
 		{
 		  stmp->lastr = time(NULL);
@@ -832,17 +849,27 @@ sic_select()
 			}
 		      continue;
 		    }
-		  else if (!option(stmp->sopt, S_DCC)
-			   && !strncmp("PING ", buf, 5))
-		      vsic_write("PONG %s :%s", server->nick,
-				 (buf[4] && buf[5]) ? buf+6 : "");
 		  else
 		    {
-		      /* current window for this server */
-		      select_active(NULL, 1);
-		      parse(buf);
+		      /* readln() */
+		      if (!option(stmp->sopt, S_DCC)
+			  && !strncmp("PING ", stmp->readbuf, 5))
+			  vsic_write("PONG %s :%s", server->nick,
+				     (stmp->readbuf[4] && stmp->readbuf[5]) ?
+				     stmp->readbuf+6 : "");
+		      else
+			{
+			  /* current window for this server */
+			  select_active(NULL, 1);
+			  parse(stmp->readbuf);
+			}
+		      stmp->readbuf[0] = '\0';
 		    }
 		}
+	      else if (sz == -2)
+		{
+		  vsic_slog(LOG_DEBUG, "Incomplete line from %s", stmp->sname);
+		}
 	      else
 		{
 		  close(stmp->fd);
@@ -1012,11 +1039,17 @@ cmd_server(p)
 	    {
 	      server = stmp;
 	      if (stmp->ip || option(stmp->sopt, S_LOOKUP))
+		{
 		  if (sic_swin(1))
+		    {
 		      if (option(stmp->sopt, S_CONNECTED))
+			{
+			  /*
 			  vsic_slog(LOG_CLIENT,
 				    "--- Window now bound to %s %d [%s]",
 				    stmp->sname, stmp->port, stmp->ip);
+			  */
+			}
 		      else
 			{
 			  set_option(stmp->sopt, S_CONNECTING);
@@ -1028,8 +1061,12 @@ cmd_server(p)
 			      sic_connect(stmp);
 			    }
 			}
+		    }
+		  /*
 		  else
 		      sic_slog(LOG_CLIENT, "--- No available window.");
+		  */
+		}
 	      else
 		  vsic_slog(LOG_CLIENT, "--- Unknown IP for %s", stmp->sname);
 	    }
@@ -1214,6 +1251,8 @@ cmd_dcc(p)
 	}
       return 0;
     }
+  if (server == NULL)
+      return -5;
   if (!strncasecmp("CHAT ", p, 5))
     {
       p += 5;
