@@ -199,6 +199,7 @@ enum /* server options */
   S_SEND,		/* DCC send */
   S_LISTEN,		/* waiting for DCC */
   S_MOTD,		/* set upon 001, removed when receiving motd */
+  S_PING,		/* PING sent */
   SMAX
 };
 
@@ -221,6 +222,7 @@ struct server_
   unsigned char		protocol;	/* protocol type */
   char			umode[32];
   int			fd;
+  char			readbuf[520];
   unsigned long		size, read, sent;	/* DCC file size */
   FILE			*readf; /* incoming file */
   int			sendf; /* outgoing file */
