@@ -7,7 +7,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "@(#)$Id: window.c,v 1.56 1998/07/20 14:52:56 kalt Exp $";
+static char rcsid[] = "@(#)$Id: window.c,v 1.58 1998/08/17 20:29:00 kalt Exp $";
 #endif
 
 #include "os.h"
@@ -444,8 +444,10 @@ display_status()
       wtmp = wtmp->nextw;
     }
   
-  sprintf(buf, "sic[%d:%d] %s%s%s%s%s %s%s%s%s (%s) [%s] %s%s%s", current->ref,
-	  current->fnb, (current->llog == current->ldisp) ? "" : "*more* ",
+  sprintf(buf, "sic[%d:%d%s] %s%s%s%s%s %s%s%s%s (%s) [%s] %s%s%s",
+	  current->ref, current->fnb,
+	  (option(current->custw.zopt_on, Z_UNDEL)) ? "u" : "",
+	  (current->llog == current->ldisp) ? "" : "*more* ",
 	  (*actbuf) ? "(" : "", actbuf, (*actbuf) ? ") " : "",
 	  tbuf,
 	  (current->via && current->via->nick) ? current->via->nick : "",
@@ -525,9 +527,9 @@ char
 sic_swin(what)
   char what;
 {
-  struct window_ *wtmp = wlist;
+  struct window_ *wtmp = wlist, *act = active;
   struct server_ *olds = current->via;
-  char cnt = 0;
+  char cnt = 0, merge = 0;
 
   switch (what)
     {
@@ -536,6 +538,12 @@ sic_swin(what)
       if (option(server->sopt, S_CONNECTED)
 	  || option(server->sopt, S_CONNECTING))
 	{
+	  if (current->defchan)
+	    {
+	      sic_slog(LOG_CLIENT,
+		       "--- Part channels in current window first!");
+	      break;
+	    }
 	  current->via = server;
 	  unset_option(current->wopt, W_DEFAULT);
 	  cnt++;
@@ -545,10 +553,46 @@ sic_swin(what)
 	}
       else
 	{
+	  /* are we merging two groups of windows? */
+	  while (wtmp)
+	    {
+	      if (wtmp->via == server)
+		  merge += 1;
+	      wtmp = wtmp->nextw;
+	    }
+	  wtmp = wlist;
+	  if (merge)
+	    {
+	      /*
+	      ** merging can get really tricky because of channels,
+	      ** so only simple merge cases are allowed.
+	      */
+	      while (wtmp)
+		{
+		  if (wtmp->via == olds && wtmp->defchan)
+		      cnt += 1;
+		  wtmp = wtmp->nextw;
+		}
+	      if (cnt)
+		{
+		  vsic_slog(LOG_CLIENT, "--- %d window(s) from this group have channels bound to them!", cnt);
+		  break;
+		}
+	      wtmp = wlist;
+	    }
 	  while (wtmp)
 	    {
 	      if (wtmp->via == olds)
 		{
+		  if (server)
+		    {
+		      active = wtmp;
+		      vsic_slog(LOG_CLIENT,
+				"--- Window now bound to %s %d [%s]",
+				server->sname, server->port,
+				(server->ip) ? server->ip : "???");
+		    }
+		  active = current;
 		  vsic_slog(LOG_DEBUG, "(re)assigned window %d from %s to %s.",
 			    wtmp->ref, (olds) ? olds->sname : "<none>",
 			    server->sname);
@@ -559,6 +603,7 @@ sic_swin(what)
 	      wtmp = wtmp->nextw;
 	    }
 	  set_option(current->wopt, W_DEFAULT);
+	  active = current;
 	  wtmp = wlist;
 	  while (wtmp)
 	    {
@@ -631,6 +676,7 @@ sic_swin(what)
   default:
       abort(); /* never */
     }
+  active = act;
   return cnt;
 }
 
